from PyQt5.QtWidgets import QMainWindow, QApplication, QMessageBox, QScrollArea, QComboBox, QFrame, QCheckBox, QFileDialog, QGridLayout, QTextEdit, QLineEdit, QVBoxLayout, QWidget, QLabel, QHBoxLayout, QPushButton
from PyQt5.QtCore import Qt, QSize
from celldetective.gui.gui_utils import center_window, FeatureChoice, ListWidget, QHSeperationLine, FigureCanvas, help_generic
from superqt import QLabeledDoubleSlider,QLabeledSlider
from superqt.fonticon import icon
from fonticon_mdi6 import MDI6
from celldetective.utils import extract_experiment_channels, get_software_location
from celldetective.io import interpret_tracking_configuration, load_frames
from celldetective.measure import compute_haralick_features
import numpy as np
import json
from shutil import copyfile
import os
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable
from glob import glob
from celldetective.gui import Styles

class ConfigTracking(QMainWindow, Styles):
	
	"""
	UI to set tracking parameters for bTrack.

	"""

	def __init__(self, parent_window=None):
		
		super().__init__()
		self.parent_window = parent_window
		self.setWindowTitle("Configure tracking")
		self.mode = self.parent_window.mode
		self.exp_dir = self.parent_window.exp_dir
		if self.mode=="targets":
			self.config_name = os.sep.join(["configs", "btrack_config_targets.json"])
			self.track_instructions_write_path = self.parent_window.exp_dir + os.sep.join(["configs","tracking_instructions_targets.json"])
		elif self.mode=="effectors":
			self.config_name = os.sep.join(["configs","btrack_config_effectors.json"])
			self.track_instructions_write_path = self.parent_window.exp_dir + os.sep.join(["configs", "tracking_instructions_effectors.json"])
		self.soft_path = get_software_location()
		
		exp_config = self.exp_dir +"config.ini"
		self.config_path = self.exp_dir + self.config_name
		self.channel_names, self.channels = extract_experiment_channels(exp_config)
		self.channel_names = np.array(self.channel_names)
		self.channels = np.array(self.channels)
		self.screen_height = self.parent_window.parent_window.parent_window.screen_height

		center_window(self)
		self.setMinimumWidth(540)
		self.minimum_height = 300
		# self.setMinimumHeight(int(0.3*self.screen_height))
		# self.setMaximumHeight(int(0.8*self.screen_height))
		self.populate_widget()
		self.load_previous_tracking_instructions()

	def populate_widget(self):

		"""
		Create the multibox design with collapsable frames.

		"""
		
		# Create button widget and layout
		self.scroll_area = QScrollArea(self)
		self.button_widget = QWidget()
		main_layout = QVBoxLayout()
		self.button_widget.setLayout(main_layout)
		main_layout.setContentsMargins(30,30,30,30)

		# First collapsable Frame CONFIG
		self.config_frame = QFrame()
		self.config_frame.setFrameStyle(QFrame.StyledPanel | QFrame.Raised)
		self.populate_config_frame()
		main_layout.addWidget(self.config_frame)

		# Second collapsable frame FEATURES
		self.features_frame = QFrame()
		self.features_frame.setFrameStyle(QFrame.StyledPanel | QFrame.Raised)
		self.populate_features_frame()
		main_layout.addWidget(self.features_frame)

		# Third collapsable frame POST-PROCESSING
		self.post_proc_frame = QFrame()
		self.post_proc_frame.setFrameStyle(QFrame.StyledPanel | QFrame.Raised)
		self.populate_post_proc_frame()
		main_layout.addWidget(self.post_proc_frame)

		self.submit_btn = QPushButton('Save')
		self.submit_btn.setStyleSheet(self.parent_window.parent_window.parent_window.button_style_sheet)
		self.submit_btn.clicked.connect(self.write_instructions)
		main_layout.addWidget(self.submit_btn)

		#self.populate_left_panel()
		#grid.addLayout(self.left_side, 0, 0, 1, 1)
		self.button_widget.adjustSize()

		self.scroll_area.setAlignment(Qt.AlignCenter)
		self.scroll_area.setWidget(self.button_widget)
		self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
		self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
		self.scroll_area.setWidgetResizable(True)
		self.setCentralWidget(self.scroll_area)
		self.show()

		QApplication.processEvents()
		self.adjustScrollArea()


	def populate_post_proc_frame(self):

		"""
		Add widgets and layout in the POST-PROCESSING frame.
		"""

		grid = QGridLayout(self.post_proc_frame)

		self.select_post_proc_btn = QPushButton()
		self.select_post_proc_btn.clicked.connect(self.activate_post_proc_options)
		self.select_post_proc_btn.setStyleSheet(self.button_select_all)

		self.post_proc_lbl = QLabel("POST-PROCESSING")
		self.post_proc_lbl.setStyleSheet("""
			font-weight: bold;
			padding: 0px;
			""")
		grid.addWidget(self.post_proc_lbl, 0, 0, 1, 4, alignment=Qt.AlignCenter)

		title_hbox = QHBoxLayout()

		self.collapse_post_proc_btn = QPushButton()
		self.collapse_post_proc_btn.setIcon(icon(MDI6.chevron_down,color="black"))
		self.collapse_post_proc_btn.setIconSize(QSize(20, 20))
		self.collapse_post_proc_btn.setStyleSheet(self.button_select_all)
		#grid.addWidget(self.collapse_post_proc_btn, 0, 0, 1, 4, alignment=Qt.AlignRight)

		self.help_post_btn = QPushButton()
		self.help_post_btn.setIcon(icon(MDI6.help_circle,color=self.help_color))
		self.help_post_btn.setIconSize(QSize(20, 20))
		self.help_post_btn.clicked.connect(self.help_post)
		self.help_post_btn.setStyleSheet(self.button_select_all)
		self.help_post_btn.setToolTip("Help.")

		title_hbox.addWidget(self.select_post_proc_btn, 5)
		title_hbox.addWidget(QLabel(), 85, alignment=Qt.AlignCenter)
		title_hbox.addWidget(self.help_post_btn, 5)
		title_hbox.addWidget(self.collapse_post_proc_btn, 5)
		grid.addLayout(title_hbox, 0,0,1,4)

		self.generate_post_proc_panel_contents()
		grid.addWidget(self.ContentsPostProc, 1, 0, 1, 4, alignment=Qt.AlignTop)
		self.collapse_post_proc_btn.clicked.connect(lambda: self.ContentsPostProc.setHidden(not self.ContentsPostProc.isHidden()))
		self.collapse_post_proc_btn.clicked.connect(self.collapse_post_advanced)
		self.ContentsPostProc.hide()
		self.uncheck_post_proc()

	def collapse_post_advanced(self):

		features_open = not self.ContentsFeatures.isHidden()
		config_open = not self.ContentsConfig.isHidden()
		post_open = not self.ContentsPostProc.isHidden()
		is_open = np.array([features_open, config_open, post_open])

		if self.ContentsPostProc.isHidden():
			self.collapse_post_proc_btn.setIcon(icon(MDI6.chevron_down,color="black"))
			self.collapse_post_proc_btn.setIconSize(QSize(20, 20))
			if len(is_open[is_open])==0:
				self.scroll_area.setMinimumHeight(int(self.minimum_height))
				self.adjustSize()
		else:
			self.collapse_post_proc_btn.setIcon(icon(MDI6.chevron_up,color="black"))
			self.collapse_post_proc_btn.setIconSize(QSize(20, 20))
			self.scroll_area.setMinimumHeight(min(int(930), int(0.9*self.screen_height)))


	def help_post(self):
		
		"""
		Helper for track post-processing strategy.
		"""

		dict_path = os.sep.join([get_software_location(),'celldetective','gui','help','track-postprocessing.json'])

		with open(dict_path) as f:
			d = json.load(f)

		suggestion = help_generic(d)
		if isinstance(suggestion, str):
			print(f"{suggestion=}")
			msgBox = QMessageBox()
			msgBox.setIcon(QMessageBox.Information)
			msgBox.setTextFormat(Qt.RichText)
			msgBox.setText(rf"{suggestion}")
			msgBox.setWindowTitle("Info")
			msgBox.setStandardButtons(QMessageBox.Ok)
			returnValue = msgBox.exec()
			if returnValue == QMessageBox.Ok:
				return None		

	def help_feature(self):
		
		"""
		Helper for track post-processing strategy.
		"""

		dict_path = os.sep.join([get_software_location(),'celldetective','gui','help','feature-btrack.json'])

		with open(dict_path) as f:
			d = json.load(f)

		suggestion = help_generic(d)
		if isinstance(suggestion, str):
			print(f"{suggestion=}")
			msgBox = QMessageBox()
			msgBox.setIcon(QMessageBox.Information)
			msgBox.setTextFormat(Qt.RichText)
			msgBox.setText(rf"{suggestion}")
			msgBox.setWindowTitle("Info")
			msgBox.setStandardButtons(QMessageBox.Ok)
			returnValue = msgBox.exec()
			if returnValue == QMessageBox.Ok:
				return None

	def populate_features_frame(self):

		"""
		Add widgets and layout in the FEATURES frame.
		"""

		grid = QGridLayout(self.features_frame)
		title_hbox = QHBoxLayout()
		
		self.select_features_btn = QPushButton()
		self.select_features_btn.clicked.connect(self.activate_feature_options)
		self.select_features_btn.setStyleSheet(self.button_select_all)

		self.feature_lbl = QLabel("FEATURES")
		self.feature_lbl.setStyleSheet("""
			font-weight: bold;
			padding: 0px;
			""")
		grid.addWidget(self.feature_lbl, 0, 0, 1, 4, alignment=Qt.AlignCenter)

		self.collapse_features_btn = QPushButton()
		self.collapse_features_btn.setIcon(icon(MDI6.chevron_down,color="black"))
		self.collapse_features_btn.setIconSize(QSize(20, 20))
		self.collapse_features_btn.setStyleSheet(self.button_select_all)

		self.help_feature_btn = QPushButton()
		self.help_feature_btn.setIcon(icon(MDI6.help_circle,color=self.help_color))
		self.help_feature_btn.setIconSize(QSize(20, 20))
		self.help_feature_btn.clicked.connect(self.help_feature)
		self.help_feature_btn.setStyleSheet(self.button_select_all)
		self.help_feature_btn.setToolTip("Help.")

		title_hbox.addWidget(self.select_features_btn, 5)
		title_hbox.addWidget(QLabel(), 85, alignment=Qt.AlignCenter)
		title_hbox.addWidget(self.help_feature_btn, 5)
		title_hbox.addWidget(self.collapse_features_btn, 5)
		grid.addLayout(title_hbox, 0,0,1,4)		

		self.generate_feature_panel_contents()
		grid.addWidget(self.ContentsFeatures, 1, 0, 1, 4, alignment=Qt.AlignTop)
		self.collapse_features_btn.clicked.connect(lambda: self.ContentsFeatures.setHidden(not self.ContentsFeatures.isHidden()))
		self.collapse_features_btn.clicked.connect(self.collapse_features_advanced)
		#self.ContentsFeatures.hide()
		self.check_features()

	def collapse_features_advanced(self):

		"""
		Switch the chevron icon and adjust the size for the FEATURES frame.
		"""

		features_open = not self.ContentsFeatures.isHidden()
		config_open = not self.ContentsConfig.isHidden()
		post_open = not self.ContentsPostProc.isHidden()
		is_open = np.array([features_open, config_open, post_open])

		if self.ContentsFeatures.isHidden():
			self.collapse_features_btn.setIcon(icon(MDI6.chevron_down,color="black"))
			self.collapse_features_btn.setIconSize(QSize(20, 20))
			if len(is_open[is_open])==0:
				self.scroll_area.setMinimumHeight(int(self.minimum_height))
				self.adjustSize()
		else:
			self.collapse_features_btn.setIcon(icon(MDI6.chevron_up,color="black"))
			self.collapse_features_btn.setIconSize(QSize(20, 20))
			self.scroll_area.setMinimumHeight(min(int(930), int(0.9*self.screen_height)))


	def generate_post_proc_panel_contents(self):

		self.ContentsPostProc = QFrame()
		layout = QVBoxLayout(self.ContentsPostProc)
		layout.setContentsMargins(0,0,0,0)

		post_proc_layout  = QHBoxLayout()
		self.post_proc_lbl = QLabel("Post processing on the tracks:")
		post_proc_layout.addWidget(self.post_proc_lbl, 90)
		layout.addLayout(post_proc_layout)

		clean_traj_sublayout = QVBoxLayout()
		clean_traj_sublayout.setContentsMargins(15,15,15,15)

		tracklength_layout = QHBoxLayout()
		self.min_tracklength_slider = QLabeledSlider()
		self.min_tracklength_slider.setSingleStep(1)
		self.min_tracklength_slider.setTickInterval(1)
		self.min_tracklength_slider.setSingleStep(1)
		self.min_tracklength_slider.setOrientation(1)
		self.min_tracklength_slider.setRange(0,self.parent_window.parent_window.len_movie)
		self.min_tracklength_slider.setValue(0)
		tracklength_layout.addWidget(QLabel('Min. tracklength: '),40)
		tracklength_layout.addWidget(self.min_tracklength_slider, 60)
		clean_traj_sublayout.addLayout(tracklength_layout)

		self.remove_not_in_first_checkbox = QCheckBox('Remove tracks that do not start at the beginning')
		self.remove_not_in_first_checkbox.setIcon(icon(MDI6.arrow_expand_right,color="k"))
		clean_traj_sublayout.addWidget(self.remove_not_in_first_checkbox)

		self.remove_not_in_last_checkbox = QCheckBox('Remove tracks that do not end at the end')
		self.remove_not_in_last_checkbox.setIcon(icon(MDI6.arrow_expand_left,color="k"))
		clean_traj_sublayout.addWidget(self.remove_not_in_last_checkbox)

		self.interpolate_gaps_checkbox = QCheckBox('Interpolate missed detections within tracks')
		self.interpolate_gaps_checkbox.setIcon(icon(MDI6.chart_timeline_variant_shimmer,color="k"))
		clean_traj_sublayout.addWidget(self.interpolate_gaps_checkbox)

		self.extrapolate_post_checkbox = QCheckBox('Sustain last position until the end of the movie')
		self.extrapolate_post_checkbox.setIcon(icon(MDI6.repeat,color="k"))

		self.extrapolate_pre_checkbox = QCheckBox('Sustain first position from the beginning of the movie')
		self.extrapolate_pre_checkbox.setIcon(icon(MDI6.repeat,color="k"))

		clean_traj_sublayout.addWidget(self.extrapolate_post_checkbox)
		clean_traj_sublayout.addWidget(self.extrapolate_pre_checkbox)

		self.interpolate_na_features_checkbox = QCheckBox('Interpolate features of missed detections')
		self.interpolate_na_features_checkbox.setIcon(icon(MDI6.format_color_fill,color="k"))

		clean_traj_sublayout.addWidget(self.interpolate_na_features_checkbox)
		clean_traj_sublayout.addStretch()

		self.post_proc_options_to_disable = [self.post_proc_lbl, self.min_tracklength_slider, self.remove_not_in_first_checkbox,
											self.remove_not_in_last_checkbox, self.interpolate_gaps_checkbox, self.extrapolate_post_checkbox,
											self.extrapolate_pre_checkbox, self.interpolate_na_features_checkbox]

		layout.addLayout(clean_traj_sublayout)


	def generate_feature_panel_contents(self):
		
		self.ContentsFeatures = QFrame()
		layout = QVBoxLayout(self.ContentsFeatures)
		layout.setContentsMargins(0,0,0,0)

		feature_layout = QHBoxLayout()
		feature_layout.setContentsMargins(0,0,0,0)


		self.feature_lbl = QLabel("Add features:")
		self.del_feature_btn = QPushButton("")
		self.del_feature_btn.setStyleSheet(self.button_select_all)
		self.del_feature_btn.setIcon(icon(MDI6.trash_can,color="black"))
		self.del_feature_btn.setToolTip("Remove feature")
		self.del_feature_btn.setIconSize(QSize(20, 20))

		self.add_feature_btn = QPushButton("")
		self.add_feature_btn.setStyleSheet(self.button_select_all)
		self.add_feature_btn.setIcon(icon(MDI6.filter_plus,color="black"))
		self.add_feature_btn.setToolTip("Add feature")
		self.add_feature_btn.setIconSize(QSize(20, 20))		

		self.features_list = ListWidget(FeatureChoice, initial_features=['area','intensity_mean',])

		self.del_feature_btn.clicked.connect(self.features_list.removeSel)
		self.add_feature_btn.clicked.connect(self.features_list.addItem)

		feature_layout.addWidget(self.feature_lbl, 90)
		feature_layout.addWidget(self.del_feature_btn, 5)
		feature_layout.addWidget(self.add_feature_btn, 5)
		layout.addLayout(feature_layout)
		layout.addWidget(self.features_list)

		self.feat_sep2 = QHSeperationLine()
		layout.addWidget(self.feat_sep2)

		self.use_channel_lbl = QLabel('Use channel:')
		layout.addWidget(self.use_channel_lbl)
		self.mask_channels_cb = [QCheckBox() for i in range(len(self.channels))]
		for cb,chn in zip(self.mask_channels_cb,self.channel_names):
			cb.setText(chn)
			cb.setChecked(True)
			layout.addWidget(cb)

		self.feat_sep1 = QHSeperationLine()
		layout.addWidget(self.feat_sep1)

		self.activate_haralick_btn = QCheckBox('activate Haralick texture features')
		self.activate_haralick_btn.toggled.connect(self.show_haralick_options)
		# Haralick features parameters

		self.haralick_channel_choice = QComboBox()
		self.haralick_channel_choice.addItems(self.channel_names)
		self.haralick_channel_lbl = QLabel('Target channel: ')

		self.haralick_distance_le = QLineEdit("1")
		self.haralick_distance_lbl = QLabel('Distance: ')

		self.haralick_n_gray_levels_le = QLineEdit("256")
		self.haralick_n_gray_levels_lbl = QLabel('# gray levels: ')

		# Slider to set vmin & vmax
		self.haralick_scale_slider = QLabeledDoubleSlider()
		self.haralick_scale_slider.setSingleStep(0.05)
		self.haralick_scale_slider.setTickInterval(0.05)
		self.haralick_scale_slider.setSingleStep(1)
		self.haralick_scale_slider.setOrientation(1)
		self.haralick_scale_slider.setRange(0,1)
		self.haralick_scale_slider.setValue(0.5)
		self.haralick_scale_lbl = QLabel('Scale: ')

		self.haralick_percentile_min_le = QLineEdit('0.01')
		self.haralick_percentile_max_le = QLineEdit('99.9')
		self.haralick_normalization_mode_btn = QPushButton()
		self.haralick_normalization_mode_btn.setStyleSheet(self.button_select_all)
		self.haralick_normalization_mode_btn.setIcon(icon(MDI6.percent_circle,color="black"))
		self.haralick_normalization_mode_btn.setIconSize(QSize(20, 20))		
		self.haralick_normalization_mode_btn.setToolTip("Switch to absolute normalization values.")
		self.percentile_mode = True

		self.haralick_percentile_min_lbl = QLabel('Min percentile: ')
		self.haralick_percentile_max_lbl = QLabel('Max percentile: ')

		self.haralick_hist_btn = QPushButton()
		self.haralick_hist_btn.clicked.connect(self.control_haralick_intensity_histogram)
		self.haralick_hist_btn.setIcon(icon(MDI6.poll,color="k"))
		self.haralick_hist_btn.setStyleSheet(self.button_select_all)

		self.haralick_digit_btn = QPushButton()
		self.haralick_digit_btn.clicked.connect(self.control_haralick_digitalization)
		self.haralick_digit_btn.setIcon(icon(MDI6.image_check,color="k"))
		self.haralick_digit_btn.setStyleSheet(self.button_select_all)

		self.haralick_layout = QVBoxLayout()
		self.haralick_layout.setContentsMargins(20,20,20,20)

		activate_layout = QHBoxLayout()
		activate_layout.addWidget(self.activate_haralick_btn, 80)
		activate_layout.addWidget(self.haralick_hist_btn, 10)
		activate_layout.addWidget(self.haralick_digit_btn, 10)
		self.haralick_layout.addLayout(activate_layout)

		channel_layout = QHBoxLayout()
		channel_layout.addWidget(self.haralick_channel_lbl, 40)
		channel_layout.addWidget(self.haralick_channel_choice, 60)
		self.haralick_layout.addLayout(channel_layout)

		distance_layout = QHBoxLayout()
		distance_layout.addWidget(self.haralick_distance_lbl,40)
		distance_layout.addWidget(self.haralick_distance_le, 60)
		self.haralick_layout.addLayout(distance_layout)

		gl_layout = QHBoxLayout()
		gl_layout.addWidget(self.haralick_n_gray_levels_lbl,40)
		gl_layout.addWidget(self.haralick_n_gray_levels_le,60)
		self.haralick_layout.addLayout(gl_layout)

		slider_layout = QHBoxLayout()
		slider_layout.addWidget(self.haralick_scale_lbl,40)
		slider_layout.addWidget(self.haralick_scale_slider,60)
		self.haralick_layout.addLayout(slider_layout)

		slider_min_percentile_layout = QHBoxLayout()
		slider_min_percentile_layout.addWidget(self.haralick_percentile_min_lbl,40)
		slider_min_percentile_layout.addWidget(self.haralick_percentile_min_le,55)
		slider_min_percentile_layout.addWidget(self.haralick_normalization_mode_btn, 5)
		self.haralick_layout.addLayout(slider_min_percentile_layout)

		slider_max_percentile_layout = QHBoxLayout()
		slider_max_percentile_layout.addWidget(self.haralick_percentile_max_lbl,40)
		slider_max_percentile_layout.addWidget(self.haralick_percentile_max_le,60)
		self.haralick_layout.addLayout(slider_max_percentile_layout)

		self.haralick_to_hide = [self.haralick_hist_btn, self.haralick_digit_btn, self.haralick_channel_lbl, self.haralick_channel_choice,
								self.haralick_distance_le, self.haralick_distance_lbl, self.haralick_n_gray_levels_le, self.haralick_n_gray_levels_lbl,
								self.haralick_scale_lbl, self.haralick_scale_slider, self.haralick_percentile_min_lbl, self.haralick_percentile_min_le,
								self.haralick_percentile_max_lbl, self.haralick_percentile_max_le, self.haralick_normalization_mode_btn]
		for element in self.haralick_to_hide:
			element.hide()

		self.features_to_disable = [self.feature_lbl, self.del_feature_btn, self.add_feature_btn, self.features_list, 
									self.use_channel_lbl, *self.mask_channels_cb, self.activate_haralick_btn]

		self.haralick_normalization_mode_btn.clicked.connect(self.switch_to_absolute_normalization_mode)
		layout.addLayout(self.haralick_layout)

	def switch_to_absolute_normalization_mode(self):
		if self.percentile_mode:
			self.percentile_mode = False
			self.haralick_normalization_mode_btn.setIcon(icon(MDI6.percent_circle_outline,color="black"))
			self.haralick_normalization_mode_btn.setIconSize(QSize(20, 20))		
			self.haralick_normalization_mode_btn.setToolTip("Switch to percentile normalization values.")			
			self.haralick_percentile_min_lbl.setText('Min value: ')
			self.haralick_percentile_max_lbl.setText('Max value: ')
			self.haralick_percentile_min_le.setText('0')
			self.haralick_percentile_max_le.setText('10000')

		else:
			self.percentile_mode = True
			self.haralick_normalization_mode_btn.setIcon(icon(MDI6.percent_circle,color="black"))
			self.haralick_normalization_mode_btn.setIconSize(QSize(20, 20))
			self.haralick_normalization_mode_btn.setToolTip("Switch to absolute normalization values.")			
			self.haralick_percentile_min_lbl.setText('Min percentile: ')
			self.haralick_percentile_max_lbl.setText('Max percentile: ')			
			self.haralick_percentile_min_le.setText('0.01')
			self.haralick_percentile_max_le.setText('99.99')

	def populate_config_frame(self):

		grid = QGridLayout(self.config_frame)
		panel_title = QLabel(f"CONFIGURATION")
		panel_title.setStyleSheet("""
			font-weight: bold;
			padding: 0px;
			""")

		grid.addWidget(panel_title, 0, 0, 1, 4, alignment=Qt.AlignCenter)

		self.collapse_config_btn = QPushButton()
		self.collapse_config_btn.setIcon(icon(MDI6.chevron_down,color="black"))
		self.collapse_config_btn.setIconSize(QSize(20, 20))
		self.collapse_config_btn.setStyleSheet(self.button_select_all)
		grid.addWidget(self.collapse_config_btn, 0, 0, 1, 4, alignment=Qt.AlignRight)

		self.generate_config_panel_contents()
		grid.addWidget(self.ContentsConfig, 1, 0, 1, 4, alignment=Qt.AlignTop)
		self.collapse_config_btn.clicked.connect(lambda: self.ContentsConfig.setHidden(not self.ContentsConfig.isHidden()))
		self.collapse_config_btn.clicked.connect(self.collapse_config_advanced)
		#self.ContentsConfig.hide()

	def collapse_config_advanced(self):

		"""
		Switch the chevron icon and adjust the size for the CONFIG frame.
		"""

		features_open = not self.ContentsFeatures.isHidden()
		config_open = not self.ContentsConfig.isHidden()
		post_open = not self.ContentsPostProc.isHidden()
		is_open = np.array([features_open, config_open, post_open])

		if self.ContentsConfig.isHidden():
			self.collapse_config_btn.setIcon(icon(MDI6.chevron_down,color="black"))
			self.collapse_config_btn.setIconSize(QSize(20, 20))
			if len(is_open[is_open])==0:
				self.scroll_area.setMinimumHeight(int(self.minimum_height))
				self.adjustSize()
		else:
			self.collapse_config_btn.setIcon(icon(MDI6.chevron_up,color="black"))
			self.collapse_config_btn.setIconSize(QSize(20, 20))
			self.scroll_area.setMinimumHeight(min(int(930), int(0.9*self.screen_height)))


	def generate_config_panel_contents(self):
		
		self.ContentsConfig = QFrame()
		layout = QVBoxLayout(self.ContentsConfig)
		layout.setContentsMargins(0,0,0,0)

		btrack_config_layout = QHBoxLayout()
		self.config_lbl = QLabel("bTrack configuration: ")
		btrack_config_layout.addWidget(self.config_lbl, 90)

		self.upload_btrack_config_btn = QPushButton()
		self.upload_btrack_config_btn.setIcon(icon(MDI6.plus,color="black"))
		self.upload_btrack_config_btn.setIconSize(QSize(20, 20))
		self.upload_btrack_config_btn.setToolTip("Upload a new bTrack configuration.")
		self.upload_btrack_config_btn.setStyleSheet(self.button_select_all)
		self.upload_btrack_config_btn.clicked.connect(self.upload_btrack_config)
		btrack_config_layout.addWidget(self.upload_btrack_config_btn, 5)  #4,3,1,1, alignment=Qt.AlignLeft

		self.reset_config_btn = QPushButton()
		self.reset_config_btn.setIcon(icon(MDI6.arrow_u_right_top,color="black"))
		self.reset_config_btn.setIconSize(QSize(20, 20))
		self.reset_config_btn.setToolTip("Reset the configuration to the default bTrack config.")
		self.reset_config_btn.setStyleSheet(self.button_select_all)
		self.reset_config_btn.clicked.connect(self.reset_btrack_config)
		btrack_config_layout.addWidget(self.reset_config_btn, 5)  #4,3,1,1, alignment=Qt.AlignLeft

		layout.addLayout(btrack_config_layout)

		self.config_le = QTextEdit()
		self.config_le.setMinimumHeight(150)
		#self.config_le.setStyleSheet("""
		#							background: #EEEDEB;
		#							border: 2px solid black;
		#							""")
		layout.addWidget(self.config_le)
		self.load_cell_config()


	def show_haralick_options(self):

		"""
		Show the Haralick texture options.
		"""

		if self.activate_haralick_btn.isChecked():
			for element in self.haralick_to_hide:
				element.show()
		else:
			for element in self.haralick_to_hide:
				element.hide()   		

	def upload_btrack_config(self):

		"""
		Upload a specific bTrack config to the experiment folder for the cell population.
		"""

		self.file_dialog = QFileDialog()
		try:
			modelpath = os.sep.join([self.soft_path, "celldetective","models","tracking_configs"]) + os.sep
			print("Track config path: ", modelpath)
			self.filename = self.file_dialog.getOpenFileName(None, "Load config", modelpath, "json files (*.json)")[0]
			if self.filename!=self.config_path:
				copyfile(self.filename, self.config_path)
			self.load_cell_config()
		except Exception as e:
			print(e, modelpath)
			return None

	def reset_btrack_config(self):

		"""
		Set the bTrack config to the default bTrack config.
		"""

		msgBox = QMessageBox()
		msgBox.setIcon(QMessageBox.Question)
		msgBox.setText("You are about to revert to the default bTrack configuration? Do you want to proceed?")
		msgBox.setWindowTitle("Confirm")
		msgBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
		returnValue = msgBox.exec()
		if returnValue == QMessageBox.Yes:
			config = interpret_tracking_configuration(None)
			if config!=self.config_path:
				copyfile(config, self.config_path)
			self.load_cell_config()
		else:
			return None

	def activate_feature_options(self):

		"""
		Tick the features option.
		"""

		self.switch_feature_option()
		if self.features_ticked:
			for element in self.features_to_disable:
				element.setEnabled(True)
			self.select_features_btn.setIcon(icon(MDI6.checkbox_outline,color="black"))
			self.select_features_btn.setIconSize(QSize(20, 20))
		else:
			for element in self.features_to_disable:
				element.setEnabled(False)
			self.select_features_btn.setIcon(icon(MDI6.checkbox_blank_outline,color="black"))
			self.select_features_btn.setIconSize(QSize(20, 20))
			self.features_list.list_widget.clearSelection()
			self.activate_haralick_btn.setChecked(False)


	def activate_post_proc_options(self):

		"""
		Tick the features option.
		"""

		self.switch_post_proc_option()
		if self.post_proc_ticked:
			for element in self.post_proc_options_to_disable:
				element.setEnabled(True)
			self.select_post_proc_btn.setIcon(icon(MDI6.checkbox_outline,color="black"))
			self.select_post_proc_btn.setIconSize(QSize(20, 20))
		else:
			for element in self.post_proc_options_to_disable:
				element.setEnabled(False)
			self.select_post_proc_btn.setIcon(icon(MDI6.checkbox_blank_outline,color="black"))
			self.select_post_proc_btn.setIconSize(QSize(20, 20))

	def switch_feature_option(self):

		"""
		Switch the feature option.
		"""

		if self.features_ticked == True:
			self.features_ticked = False
		else:
			self.features_ticked = True

	def switch_post_proc_option(self):

		"""
		Switch the feature option.
		"""

		if self.post_proc_ticked == True:
			self.post_proc_ticked = False
		else:
			self.post_proc_ticked = True	

	def adjustScrollArea(self):
		
		"""
		Auto-adjust scroll area to fill space 
		(from https://stackoverflow.com/questions/66417576/make-qscrollarea-use-all-available-space-of-qmainwindow-height-axis)
		"""

		step = 5
		while self.scroll_area.verticalScrollBar().isVisible() and self.height() < self.maximumHeight():
			self.resize(self.width(), self.height() + step)

	def load_cell_config(self):

		"""
		Load the cell configuration and write in the QLineEdit.
		"""

		file_name = interpret_tracking_configuration(self.config_path)
		with open(file_name, 'r') as f:
			json_data = json.load(f)
			self.config_le.setText(json.dumps(json_data, indent=4))

	def write_instructions(self):

		"""
		Write the selected options in a json file for later reading by the software.
		"""

		print('Writing instructions...')
		tracking_options = {'btrack_config_path': self.config_path}
		if not self.features_ticked:
			features = None
			masked_channels = None
		else:
			features = self.features_list.getItems()
			masked_channels = self.channel_names[np.array([not cb.isChecked() for cb in self.mask_channels_cb])]
			if len(masked_channels)==0:
				masked_channels = None
			else:
				masked_channels = list(masked_channels)

		tracking_options.update({'features': features, 'mask_channels': masked_channels})
		
		self.extract_haralick_options()
		tracking_options.update({'haralick_options': self.haralick_options})

		if self.post_proc_ticked:
			post_processing_options = {"minimum_tracklength": int(self.min_tracklength_slider.value()),
									   "remove_not_in_first": self.remove_not_in_first_checkbox.isChecked(), 
									   "remove_not_in_last": self.remove_not_in_last_checkbox.isChecked(),
									   "interpolate_position_gaps": self.interpolate_gaps_checkbox.isChecked(), 
									   "extrapolate_tracks_pre": self.extrapolate_pre_checkbox.isChecked(),
									   "extrapolate_tracks_post": self.extrapolate_post_checkbox.isChecked(),
									   'interpolate_na': self.interpolate_na_features_checkbox.isChecked()
									   }
		else:

			post_processing_options = None

		tracking_options.update({'post_processing_options': post_processing_options})
		file_name = self.track_instructions_write_path
		with open(file_name, 'w') as f:
			json.dump(tracking_options, f, indent=4)

		# Save the JSON data to the file
		file_name = self.config_path
		with open(file_name, 'w') as f:
			f.write(self.config_le.toPlainText())
		print('Done.')
		self.close()

	def uncheck_post_proc(self):
		self.select_post_proc_btn.setIcon(icon(MDI6.checkbox_blank_outline,color="black"))
		self.select_post_proc_btn.setIconSize(QSize(20, 20))
		self.post_proc_ticked = False
		for element in self.post_proc_options_to_disable:
			element.setEnabled(False)

	def check_post_proc(self):
		self.select_post_proc_btn.setIcon(icon(MDI6.checkbox_outline,color="black"))
		self.select_post_proc_btn.setIconSize(QSize(20, 20))
		self.post_proc_ticked = True		
		for element in self.post_proc_options_to_disable:
			element.setEnabled(True)

	def uncheck_features(self):
		self.select_features_btn.setIcon(icon(MDI6.checkbox_blank_outline,color="black"))
		self.select_features_btn.setIconSize(QSize(20, 20))
		self.features_ticked = False
		for element in self.features_to_disable:
			element.setEnabled(False)

	def check_features(self):
		self.select_features_btn.setIcon(icon(MDI6.checkbox_outline,color="black"))
		self.select_features_btn.setIconSize(QSize(20, 20))
		self.features_ticked = True		
		for element in self.features_to_disable:
			element.setEnabled(True)

	def extract_haralick_options(self):

		if self.activate_haralick_btn.isChecked():
			self.haralick_options = {"target_channel": self.haralick_channel_choice.currentIndex(),
								"scale_factor": float(self.haralick_scale_slider.value()),
								"n_intensity_bins": int(self.haralick_n_gray_levels_le.text()),
								"distance" : int(self.haralick_distance_le.text()),
								}
			if self.percentile_mode:
				self.haralick_options.update({"percentiles": (float(self.haralick_percentile_min_le.text()), float(self.haralick_percentile_max_le.text())), "clip_values": None})
			else:
				self.haralick_options.update({"percentiles": None, "clip_values": (float(self.haralick_percentile_min_le.text()), float(self.haralick_percentile_max_le.text()))})

		else:
			self.haralick_options = None		

	def load_previous_tracking_instructions(self):

		"""
		Read the tracking options from a previously written json file.
		"""

		print('Reading instructions..')
		if os.path.exists(self.track_instructions_write_path):
			with open(self.track_instructions_write_path, 'r') as f:
				tracking_instructions = json.load(f)
				print(tracking_instructions)
				
				# Features
				features = tracking_instructions['features']
				if (features is not None) and len(features)>0:
					self.check_features()
					self.ContentsFeatures.show()
					self.features_list.list_widget.clear()
					self.features_list.list_widget.addItems(features)
				else:
					self.ContentsFeatures.hide()
					self.uncheck_features()

				# Uncheck channels that are masked
				mask_channels = tracking_instructions['mask_channels']
				if (mask_channels is not None) and len(mask_channels)>0:
					for ch in mask_channels:
						for cb in self.mask_channels_cb:
							if cb.text()==ch:
								cb.setChecked(False)

				haralick_options = tracking_instructions['haralick_options']
				if haralick_options is None:
					self.activate_haralick_btn.setChecked(False)
					self.show_haralick_options()
				else:
					self.activate_haralick_btn.setChecked(True)
					self.show_haralick_options()
					if 'target_channel' in haralick_options:
						idx = haralick_options['target_channel']
						#idx = self.haralick_channel_choice.findText(text_to_find)
						self.haralick_channel_choice.setCurrentIndex(idx)
					if 'scale_factor' in haralick_options:
						self.haralick_scale_slider.setValue(float(haralick_options['scale_factor']))
					if ('percentiles' in haralick_options) and (haralick_options['percentiles'] is not None):
						perc = list(haralick_options['percentiles'])
						self.haralick_percentile_min_le.setText(str(perc[0]))
						self.haralick_percentile_max_le.setText(str(perc[1]))
					if ('clip_values' in haralick_options) and (haralick_options['clip_values'] is not None):
						values = list(haralick_options['clip_values'])
						self.haralick_percentile_min_le.setText(str(values[0]))
						self.haralick_percentile_max_le.setText(str(values[1]))
						self.percentile_mode=True
						self.switch_to_absolute_normalization_mode()
					if 'n_intensity_bins' in haralick_options:
						self.haralick_n_gray_levels_le.setText(str(haralick_options['n_intensity_bins']))
					if 'distance' in haralick_options:
						self.haralick_distance_le.setText(str(haralick_options['distance']))
		


				# Post processing options
				post_processing_options = tracking_instructions['post_processing_options']
				if post_processing_options is None:
					self.uncheck_post_proc()
					self.ContentsPostProc.hide()
					for element in [self.remove_not_in_last_checkbox, self.remove_not_in_first_checkbox, self.interpolate_gaps_checkbox, 
									self.extrapolate_post_checkbox, self.extrapolate_pre_checkbox, self.interpolate_na_features_checkbox]:
						element.setChecked(False)
					self.min_tracklength_slider.setValue(0)

				else:
					self.check_post_proc()
					self.ContentsPostProc.show()
					if "minimum_tracklength" in post_processing_options:
						self.min_tracklength_slider.setValue(int(post_processing_options["minimum_tracklength"]))
					if "remove_not_in_first" in post_processing_options:
						self.remove_not_in_first_checkbox.setChecked(post_processing_options["remove_not_in_first"])
					if "remove_not_in_last" in post_processing_options:
						self.remove_not_in_last_checkbox.setChecked(post_processing_options["remove_not_in_last"])
					if "interpolate_position_gaps" in post_processing_options:
						self.interpolate_gaps_checkbox.setChecked(post_processing_options["interpolate_position_gaps"])
					if "extrapolate_tracks_pre" in post_processing_options:
						self.extrapolate_pre_checkbox.setChecked(post_processing_options["extrapolate_tracks_pre"])
					if "extrapolate_tracks_post" in post_processing_options:
						self.extrapolate_post_checkbox.setChecked(post_processing_options["extrapolate_tracks_post"])
					if "interpolate_na" in post_processing_options:
						self.interpolate_na_features_checkbox.setChecked(post_processing_options["interpolate_na"])

	def locate_image(self):
		
		"""
		Load the first frame of the first movie found in the experiment folder as a sample.
		"""

		movies = glob(self.parent_window.parent_window.exp_dir + os.sep.join(["*","*","movie",self.parent_window.parent_window.movie_prefix+"*.tif"]))
		if len(movies)==0:
			msgBox = QMessageBox()
			msgBox.setIcon(QMessageBox.Warning)
			msgBox.setText("No movies are detected in the experiment folder. Cannot load an image to test Haralick.")
			msgBox.setWindowTitle("Warning")
			msgBox.setStandardButtons(QMessageBox.Ok)
			returnValue = msgBox.exec()
			if returnValue == QMessageBox.Yes:
				self.test_frame = None
				return None
		else:
			stack0 = movies[0]
			n_channels = len(self.channels)
			self.test_frame = load_frames(np.arange(n_channels), stack0, scale=None, normalize_input=False)


	def control_haralick_digitalization(self):

		"""
		Load an image for the first experiment movie found. 
		Apply the Haralick parameters and check the result of the digitization (normalization + binning of intensities).

		"""

		self.locate_image()
		self.extract_haralick_options()
		if self.test_frame is not None:
			digitized_img = compute_haralick_features(self.test_frame, np.zeros(self.test_frame.shape[:2]), 
													 channels=self.channel_names, return_digit_image_only=True,
													 **self.haralick_options
													 )

			self.fig, self.ax = plt.subplots()
			divider = make_axes_locatable(self.ax)
			cax = divider.append_axes('right', size='5%', pad=0.05)

			self.imshow_digit_window = FigureCanvas(self.fig, title="Haralick: control digitization")
			self.ax.clear()
			im = self.ax.imshow(digitized_img, cmap='gray')
			self.fig.colorbar(im, cax=cax, orientation='vertical')
			self.ax.set_xticks([])
			self.ax.set_yticks([])
			self.fig.set_facecolor('none')  # or 'None'
			self.fig.canvas.setStyleSheet("background-color: transparent;")
			self.imshow_digit_window.canvas.draw()
			self.imshow_digit_window.show()

	def control_haralick_intensity_histogram(self):

		"""
		Load an image for the first experiment movie found. 
		Apply the Haralick normalization parameters and check the normalized intensity histogram.
		
		"""

		self.locate_image()
		self.extract_haralick_options()
		if self.test_frame is not None:
			norm_img = compute_haralick_features(self.test_frame, np.zeros(self.test_frame.shape[:2]), 
													 channels=self.channel_names, return_norm_image_only=True,
													 **self.haralick_options
													 )
			self.fig, self.ax = plt.subplots(1,1,figsize=(4,3))
			self.hist_window = FigureCanvas(self.fig, title="Haralick: control digitized histogram")
			self.ax.clear()
			flat = norm_img.flatten()
			self.ax.hist(flat[flat==flat], bins=self.haralick_options['n_intensity_bins'])
			self.ax.set_xlabel('gray level value')
			self.ax.set_ylabel('#')
			plt.tight_layout()
			self.fig.set_facecolor('none')  # or 'None'
			self.fig.canvas.setStyleSheet("background-color: transparent;")
			self.hist_window.canvas.draw()
			self.hist_window.show()








from PyQt5.QtWidgets import QWidget, QLineEdit, QMessageBox, QHBoxLayout, QVBoxLayout, QPushButton, QLabel, \
	QCheckBox, QRadioButton, QButtonGroup
from PyQt5.QtCore import Qt, QSize
from superqt import QLabeledSlider,QLabeledDoubleSlider, QSearchableComboBox
from superqt.fonticon import icon
from fonticon_mdi6 import MDI6

import os
import numpy as np
import matplotlib.pyplot as plt
import json

from celldetective.gui.gui_utils import FigureCanvas, center_window, color_from_status, help_generic
from celldetective.gui import Styles
from celldetective.utils import get_software_location
from celldetective.measure import classify_cells_from_query, interpret_track_classification

class ClassifierWidget(QWidget, Styles):

	def __init__(self, parent_window):

		super().__init__()

		self.parent_window = parent_window
		self.screen_height = self.parent_window.parent_window.parent_window.screen_height
		self.screen_width = self.parent_window.parent_window.parent_window.screen_width
		self.currentAlpha = 1.0

		self.setWindowTitle("Custom classification")

		self.mode = self.parent_window.mode
		self.df = self.parent_window.df

		is_number = np.vectorize(lambda x: np.issubdtype(x, np.number))
		is_number_test = is_number(self.df.dtypes)
		self.cols = [col for t,col in zip(is_number_test,self.df.columns) if t]

		self.class_name = 'custom'
		self.name_le = QLineEdit(self.class_name)
		self.init_class()

		# Create the QComboBox and add some items
		center_window(self)

		
		layout = QVBoxLayout(self)
		layout.setContentsMargins(30,30,30,30)

		name_layout = QHBoxLayout()
		name_layout.addWidget(QLabel('class name: '), 33)
		name_layout.addWidget(self.name_le, 66)
		layout.addLayout(name_layout)

		fig_btn_hbox = QHBoxLayout()
		fig_btn_hbox.addWidget(QLabel(''), 95)
		self.project_times_btn = QPushButton('')
		self.project_times_btn.setStyleSheet(self.parent_window.parent_window.parent_window.button_select_all)
		self.project_times_btn.setIcon(icon(MDI6.math_integral,color="black"))
		self.project_times_btn.setToolTip("Project measurements at all times.")
		self.project_times_btn.setIconSize(QSize(20, 20))
		self.project_times = False
		self.project_times_btn.clicked.connect(self.switch_projection)
		fig_btn_hbox.addWidget(self.project_times_btn, 5)
		layout.addLayout(fig_btn_hbox)

		# Figure
		self.initalize_props_scatter()
		layout.addWidget(self.propscanvas)

		# slider
		self.frame_slider = QLabeledSlider()
		self.frame_slider.setSingleStep(1)
		self.frame_slider.setOrientation(1)
		self.frame_slider.setRange(0,int(self.df.FRAME.max()) - 1)
		self.frame_slider.setValue(0)
		self.currentFrame = 0

		slider_hbox = QHBoxLayout()
		slider_hbox.addWidget(QLabel('frame: '), 10)
		slider_hbox.addWidget(self.frame_slider, 90)
		layout.addLayout(slider_hbox)


		# transparency slider
		self.alpha_slider = QLabeledDoubleSlider()
		self.alpha_slider.setSingleStep(0.001)
		self.alpha_slider.setOrientation(1)
		self.alpha_slider.setRange(0,1)
		self.alpha_slider.setValue(1.0)
		self.alpha_slider.setDecimals(3)

		slider_alpha_hbox = QHBoxLayout()
		slider_alpha_hbox.addWidget(QLabel('transparency: '), 10)
		slider_alpha_hbox.addWidget(self.alpha_slider, 90)
		layout.addLayout(slider_alpha_hbox)



		self.features_cb = [QSearchableComboBox() for i in range(2)]
		self.log_btns = [QPushButton() for i in range(2)]

		for i in range(2):
			hbox_feat = QHBoxLayout()
			hbox_feat.addWidget(QLabel(f'feature {i}: '), 20)
			hbox_feat.addWidget(self.features_cb[i], 75)
			hbox_feat.addWidget(self.log_btns[i], 5)
			layout.addLayout(hbox_feat)

			self.features_cb[i].clear()
			self.features_cb[i].addItems(sorted(list(self.cols),key=str.lower))
			self.features_cb[i].currentTextChanged.connect(self.update_props_scatter)
			self.features_cb[i].setCurrentIndex(i)

			self.log_btns[i].setIcon(icon(MDI6.math_log,color="black"))
			self.log_btns[i].setStyleSheet(self.button_select_all)
			self.log_btns[i].clicked.connect(lambda ch, i=i: self.switch_to_log(i))

		hbox_classify = QHBoxLayout()
		hbox_classify.addWidget(QLabel('classify: '), 10)
		self.property_query_le = QLineEdit()
		self.property_query_le.setPlaceholderText('classify points using a query such as: area > 100 or eccentricity > 0.95')
		self.property_query_le.setToolTip('Classify points using a query on measurements.\nYou can use "and" and "or" conditions to combine\nmeasurements (e.g. "area > 100 or eccentricity > 0.95").')
		hbox_classify.addWidget(self.property_query_le, 70)
		self.submit_query_btn = QPushButton('Submit...')
		self.submit_query_btn.clicked.connect(self.apply_property_query)
		hbox_classify.addWidget(self.submit_query_btn, 20)
		layout.addLayout(hbox_classify)

		self.time_corr = QCheckBox('Time correlated')
		self.time_corr.toggled.connect(self.activate_time_corr_options)
		if "TRACK_ID" in self.df.columns:
			self.time_corr.setEnabled(True)
		else:
			self.time_corr.setEnabled(False)

		time_prop_hbox = QHBoxLayout()
		time_prop_hbox.addWidget(self.time_corr,alignment=Qt.AlignCenter)

		self.help_propagate_btn = QPushButton()
		self.help_propagate_btn.setIcon(icon(MDI6.help_circle,color=self.help_color))
		self.help_propagate_btn.setIconSize(QSize(20, 20))
		self.help_propagate_btn.clicked.connect(self.help_propagate)
		self.help_propagate_btn.setStyleSheet(self.button_select_all)
		self.help_propagate_btn.setToolTip("Help.")
		time_prop_hbox.addWidget(self.help_propagate_btn,5,alignment=Qt.AlignRight)

		layout.addLayout(time_prop_hbox)

		self.irreversible_event_btn = QRadioButton('irreversible event')
		self.unique_state_btn = QRadioButton('unique state')
		time_corr_btn_group = QButtonGroup()
		self.unique_state_btn.click()
		self.time_corr_options = [self.irreversible_event_btn, self.unique_state_btn]

		for btn in self.time_corr_options:
			time_corr_btn_group.addButton(btn)
			btn.setEnabled(False)

		time_corr_layout = QHBoxLayout()
		time_corr_layout.addWidget(self.unique_state_btn, 50, alignment=Qt.AlignCenter)
		time_corr_layout.addWidget(self.irreversible_event_btn, 50,alignment=Qt.AlignCenter)
		layout.addLayout(time_corr_layout)

		self.r2_slider = QLabeledDoubleSlider()
		self.r2_slider.setValue(0.75)
		self.r2_slider.setRange(0,1)
		self.r2_slider.setSingleStep(0.01)
		self.r2_slider.setOrientation(1)
		self.r2_label = QLabel('R2 tolerance:')
		self.r2_label.setToolTip('Minimum R2 between the fit sigmoid and the binary response to the filters to accept the event.')
		r2_threshold_layout = QHBoxLayout()
		r2_threshold_layout.addWidget(QLabel(''), 50)
		r2_threshold_layout.addWidget(self.r2_label, 15)
		r2_threshold_layout.addWidget(self.r2_slider, 35)
		layout.addLayout(r2_threshold_layout)	
		
		self.irreversible_event_btn.clicked.connect(self.activate_r2)
		self.unique_state_btn.clicked.connect(self.activate_r2)

		for wg in [self.r2_slider, self.r2_label]:
			wg.setEnabled(False)

		layout.addWidget(QLabel())


		self.submit_btn = QPushButton('apply')
		self.submit_btn.setStyleSheet(self.button_style_sheet)
		self.submit_btn.clicked.connect(self.submit_classification)
		layout.addWidget(self.submit_btn, 30)

		self.frame_slider.valueChanged.connect(self.set_frame)
		self.alpha_slider.valueChanged.connect(self.set_transparency)

	def activate_r2(self):
		if self.irreversible_event_btn.isChecked() and self.time_corr.isChecked():
			for wg in [self.r2_slider, self.r2_label]:
				wg.setEnabled(True)
		else:
			for wg in [self.r2_slider, self.r2_label]:
				wg.setEnabled(False)				

	def activate_time_corr_options(self):

		if self.time_corr.isChecked():
			for btn in self.time_corr_options:
				btn.setEnabled(True)
			if self.irreversible_event_btn.isChecked():
				for wg in [self.r2_slider, self.r2_label]:
					wg.setEnabled(True)
			else:
				for wg in [self.r2_slider, self.r2_label]:
					wg.setEnabled(False)				
		else:
			for btn in self.time_corr_options:
				btn.setEnabled(False)
			for wg in [self.r2_slider, self.r2_label]:
				wg.setEnabled(False)

	def init_class(self):

		self.class_name = 'custom'
		i=1
		while self.class_name in self.df.columns:
			self.class_name = f'custom_{i}'
			i+=1
		self.name_le.setText(self.class_name)
		self.df.loc[:,self.class_name] = 1

	def initalize_props_scatter(self):

		"""
		Define properties scatter.
		"""

		self.fig_props, self.ax_props = plt.subplots(figsize=(4,4),tight_layout=True)
		self.propscanvas = FigureCanvas(self.fig_props, interactive=True)
		self.fig_props.set_facecolor('none')
		self.fig_props.canvas.setStyleSheet("background-color: transparent;")
		self.scat_props = self.ax_props.scatter([],[], color="k", alpha=self.currentAlpha)
		self.propscanvas.canvas.draw_idle()
		self.propscanvas.canvas.setMinimumHeight(self.screen_height//5)

	def update_props_scatter(self, feature_changed=True):

		if not self.project_times:
			self.scat_props.set_offsets(self.df.loc[self.df['FRAME']==self.currentFrame,[self.features_cb[1].currentText(),self.features_cb[0].currentText()]].to_numpy())
			colors = [color_from_status(c) for c in self.df.loc[self.df['FRAME']==self.currentFrame,self.class_name].to_numpy()]
			self.scat_props.set_facecolor(colors)
			self.scat_props.set_alpha(self.currentAlpha)
			self.ax_props.set_xlabel(self.features_cb[1].currentText())
			self.ax_props.set_ylabel(self.features_cb[0].currentText())
		else:
			self.scat_props.set_offsets(self.df[[self.features_cb[1].currentText(),self.features_cb[0].currentText()]].to_numpy())
			colors = [color_from_class(c) for c in self.df[self.class_name].to_numpy()]
			self.scat_props.set_facecolor(colors)
			self.scat_props.set_alpha(self.currentAlpha)
			self.ax_props.set_xlabel(self.features_cb[1].currentText())
			self.ax_props.set_ylabel(self.features_cb[0].currentText())


		feat_x = self.features_cb[1].currentText()
		feat_y = self.features_cb[0].currentText()
		min_x = self.df.dropna(subset=feat_x)[feat_x].min()
		max_x = self.df.dropna(subset=feat_x)[feat_x].max()
		min_y = self.df.dropna(subset=feat_y)[feat_y].min()
		max_y = self.df.dropna(subset=feat_y)[feat_y].max()

		if min_x==min_x and max_x==max_x:
			self.ax_props.set_xlim(min_x, max_x)
		if min_y==min_y and max_y==max_y:
			self.ax_props.set_ylim(min_y, max_y)
		
		if feature_changed:
			self.propscanvas.canvas.toolbar.update()
		self.propscanvas.canvas.draw_idle()

	def apply_property_query(self):
		
		query = self.property_query_le.text()
		self.df = classify_cells_from_query(self.df, self.class_name, query)
		if self.df is None:
			msgBox = QMessageBox()
			msgBox.setIcon(QMessageBox.Warning)
			msgBox.setText(f"The query could not be understood. No filtering was applied. {e}")
			msgBox.setWindowTitle("Warning")
			msgBox.setStandardButtons(QMessageBox.Ok)
			returnValue = msgBox.exec()
			if returnValue == QMessageBox.Ok:
				return None

		self.update_props_scatter()

	def set_frame(self, value):
		xlim=self.ax_props.get_xlim()
		ylim=self.ax_props.get_ylim()
		self.currentFrame = value
		self.update_props_scatter(feature_changed=False)
		self.ax_props.set_xlim(xlim)
		self.ax_props.set_ylim(ylim)


	def set_transparency(self, value):
		xlim=self.ax_props.get_xlim()
		ylim=self.ax_props.get_ylim()
		self.currentAlpha = value
		#fc = self.scat_props.get_facecolors()
		#fc[:, 3] = value
		#self.scat_props.set_facecolors(fc)
		#self.propscanvas.canvas.draw_idle()
		self.update_props_scatter(feature_changed=False)
		self.ax_props.set_xlim(xlim)
		self.ax_props.set_ylim(ylim)

	def switch_projection(self):
		if self.project_times:
			self.project_times = False
			self.project_times_btn.setIcon(icon(MDI6.math_integral,color="black"))
			self.project_times_btn.setIconSize(QSize(20, 20))
			self.frame_slider.setEnabled(True)
		else:
			self.project_times = True
			self.project_times_btn.setIcon(icon(MDI6.math_integral_box,color="black"))
			self.project_times_btn.setIconSize(QSize(20, 20))
			self.frame_slider.setEnabled(False)
		self.update_props_scatter()

	def submit_classification(self):
		
		print('submit')
		self.apply_property_query()

		if self.time_corr.isChecked():
			self.class_name_user = 'class_'+self.name_le.text()
			print(f'User defined class name: {self.class_name_user}.')
			if self.class_name_user in self.df.columns:

				msgBox = QMessageBox()
				msgBox.setIcon(QMessageBox.Information)
				msgBox.setText(f"The class column {self.class_name_user} already exists in the table.\nProceeding will reclassify. Do you want to continue?")
				msgBox.setWindowTitle("Warning")
				msgBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
				returnValue = msgBox.exec()
				if returnValue == QMessageBox.Yes:
					pass
				else:
					return None

			name_map = {self.class_name: self.class_name_user}
			self.df = self.df.drop(list(set(name_map.values()) & set(self.df.columns)), axis=1).rename(columns=name_map)
			self.df.reset_index(inplace=True, drop=True)

			self.df = interpret_track_classification(self.df, self.class_name_user, irreversible_event=self.irreversible_event_btn.isChecked(), unique_state=self.unique_state_btn.isChecked(), r2_threshold=self.r2_slider.value())
		
		else:
			self.group_name_user = 'group_' + self.name_le.text()
			print(f'User defined characteristic group name: {self.group_name_user}.')
			if self.group_name_user in self.df.columns:

				msgBox = QMessageBox()
				msgBox.setIcon(QMessageBox.Information)
				msgBox.setText(
					f"The group column {self.group_name_user} already exists in the table.\nProceeding will reclassify. Do you want to continue?")
				msgBox.setWindowTitle("Warning")
				msgBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
				returnValue = msgBox.exec()
				if returnValue == QMessageBox.Yes:
					pass
				else:
					return None

			name_map = {self.class_name: self.group_name_user}
			self.df = self.df.drop(list(set(name_map.values()) & set(self.df.columns)), axis=1).rename(columns=name_map)
			print(self.df.columns)
			#self.df[self.group_name_user] = self.df[self.group_name_user].replace({0: 1, 1: 0})
			self.df.reset_index(inplace=True, drop=True)


		for pos,pos_group in self.df.groupby('position'):
			pos_group.to_csv(pos+os.sep.join(['output', 'tables', f'trajectories_{self.mode}.csv']), index=False)

		self.parent_window.parent_window.update_position_options()
		self.close()


	def help_propagate(self):

		"""
		Helper for segmentation strategy between threshold-based and Deep learning.
		"""

		dict_path = os.sep.join([get_software_location(),'celldetective','gui','help','propagate-classification.json'])

		with open(dict_path) as f:
			d = json.load(f)

		suggestion = help_generic(d)
		if isinstance(suggestion, str):
			print(f"{suggestion=}")
			msgBox = QMessageBox()
			msgBox.setIcon(QMessageBox.Information)
			msgBox.setTextFormat(Qt.RichText)
			msgBox.setText(rf"{suggestion}")
			msgBox.setWindowTitle("Info")
			msgBox.setStandardButtons(QMessageBox.Ok)
			returnValue = msgBox.exec()
			if returnValue == QMessageBox.Ok:
				return None

	def switch_to_log(self, i):

		"""
		Switch threshold histogram to log scale. Auto adjust.
		"""

		if i==1:
			try:
				if self.ax_props.get_xscale()=='linear':
					self.ax_props.set_xscale('log')
					self.log_btns[i].setIcon(icon(MDI6.math_log,color="#1565c0"))
				else:
					self.ax_props.set_xscale('linear')
					self.log_btns[i].setIcon(icon(MDI6.math_log,color="black"))
			except Exception as e:
				print(e)
		elif i==0:
			try:
				if self.ax_props.get_yscale()=='linear':
					ymin,ymax = self.ax_props.get_ylim()
					self.ax_props.set_yscale('log')
					self.log_btns[i].setIcon(icon(MDI6.math_log,color="#1565c0"))
				else:
					self.ax_props.set_yscale('linear')
					self.log_btns[i].setIcon(icon(MDI6.math_log,color="black"))
			except Exception as e:
				print(e)

		self.ax_props.autoscale()
		self.propscanvas.canvas.draw_idle()

		print('Done.')










from PyQt5.QtWidgets import QMainWindow, QApplication,QRadioButton, QMessageBox, QScrollArea, QComboBox, QFrame, QFileDialog, QGridLayout, QLineEdit, QVBoxLayout, QWidget, QLabel, QHBoxLayout, QPushButton
from PyQt5.QtCore import Qt, QSize
from PyQt5.QtGui import QDoubleValidator, QIntValidator, QIcon
from celldetective.gui.gui_utils import center_window
from celldetective.gui.layouts import ChannelNormGenerator

from superqt import QLabeledDoubleSlider,QLabeledSlider
from superqt.fonticon import icon
from fonticon_mdi6 import MDI6
from celldetective.utils import get_software_location
from celldetective.io import get_segmentation_datasets_list, locate_segmentation_dataset
from celldetective.segmentation import train_segmentation_model
from celldetective.gui.layouts import CellposeParamsWidget
import numpy as np
import json
import os
from glob import glob
from datetime import datetime
from celldetective.gui import Styles

class ConfigSegmentationModelTraining(QMainWindow, Styles):
	
	"""
	UI to set segmentation model training instructions.

	"""

	def __init__(self, parent_window=None):
		
		super().__init__()
		self.parent_window = parent_window
		self.setWindowTitle("Train segmentation model")
		self.setWindowIcon(QIcon(os.sep.join(['celldetective','icons','mexican-hat.png'])))
		self.mode = self.parent_window.mode
		self.exp_dir = self.parent_window.exp_dir
		self.soft_path = get_software_location()
		self.pretrained_model = None 
		self.dataset_folder = None
		self.software_models_dir = os.sep.join([self.soft_path, 'celldetective', 'models', f'segmentation_{self.mode}'])

		self.onlyFloat = QDoubleValidator()
		self.onlyInt = QIntValidator()

		self.screen_height = self.parent_window.parent_window.parent_window.screen_height
		center_window(self)

		self.setMinimumWidth(500)
		self.setMinimumHeight(int(0.3*self.screen_height))
		self.setMaximumHeight(int(0.8*self.screen_height))
		self.populate_widget()
		#self.load_previous_measurement_instructions()

	def populate_widget(self):

		"""
		Create the multibox design.

		"""
		
		# Create button widget and layout
		self.scroll_area = QScrollArea(self)
		self.button_widget = QWidget()
		self.main_layout = QVBoxLayout()
		self.button_widget.setLayout(self.main_layout)
		self.main_layout.setContentsMargins(30,30,30,30)

		# first frame for FEATURES
		self.model_frame = QFrame()
		self.model_frame.setFrameStyle(QFrame.StyledPanel | QFrame.Raised)
		self.populate_model_frame()
		self.main_layout.addWidget(self.model_frame)

		self.data_frame = QFrame()
		self.data_frame.setFrameStyle(QFrame.StyledPanel | QFrame.Raised)
		self.populate_data_frame()
		self.main_layout.addWidget(self.data_frame)

		self.hyper_frame = QFrame()
		self.hyper_frame.setFrameStyle(QFrame.StyledPanel | QFrame.Raised)
		self.populate_hyper_frame()
		self.main_layout.addWidget(self.hyper_frame)

		self.submit_btn = QPushButton('Train')
		self.submit_btn.setStyleSheet(self.button_style_sheet)
		self.submit_btn.clicked.connect(self.prep_model)
		self.main_layout.addWidget(self.submit_btn)
		self.submit_btn.setEnabled(False)

		self.spatial_calib_le.textChanged.connect(self.activate_train_btn)

		#self.populate_left_panel()
		#grid.addLayout(self.left_side, 0, 0, 1, 1)
		self.button_widget.adjustSize()

		self.scroll_area.setAlignment(Qt.AlignCenter)
		self.scroll_area.setWidget(self.button_widget)
		self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
		self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
		self.scroll_area.setWidgetResizable(True)
		self.setCentralWidget(self.scroll_area)
		self.show()

		QApplication.processEvents()
		self.adjustScrollArea()

	def populate_hyper_frame(self):

		"""
		Add widgets and layout in the POST-PROCESSING frame.
		"""

		grid = QGridLayout(self.hyper_frame)
		grid.setContentsMargins(30,30,30,30)
		grid.setSpacing(30)

		self.hyper_lbl = QLabel("HYPERPARAMETERS")
		self.hyper_lbl.setStyleSheet("""
			font-weight: bold;
			padding: 0px;
			""")
		grid.addWidget(self.hyper_lbl, 0, 0, 1, 4, alignment=Qt.AlignCenter)
		self.generate_hyper_contents()
		grid.addWidget(self.ContentsHyper, 1, 0, 1, 4, alignment=Qt.AlignTop)

	def generate_hyper_contents(self):

		self.ContentsHyper = QFrame()
		layout = QVBoxLayout(self.ContentsHyper)
		layout.setContentsMargins(0,0,0,0)

		lr_layout = QHBoxLayout()
		lr_layout.addWidget(QLabel('learning rate: '),30)
		self.lr_le = QLineEdit('0,0003')
		self.lr_le.setValidator(self.onlyFloat)
		lr_layout.addWidget(self.lr_le, 70)
		layout.addLayout(lr_layout)

		bs_layout = QHBoxLayout()
		bs_layout.addWidget(QLabel('batch size: '),30)
		self.bs_le = QLineEdit('8')
		self.bs_le.setValidator(self.onlyInt)
		bs_layout.addWidget(self.bs_le, 70)
		layout.addLayout(bs_layout)

		epochs_layout = QHBoxLayout()
		epochs_layout.addWidget(QLabel('# epochs: '), 30)
		self.epochs_slider = QLabeledSlider()
		self.epochs_slider.setRange(1,300)
		self.epochs_slider.setSingleStep(1)
		self.epochs_slider.setTickInterval(1)		
		self.epochs_slider.setOrientation(1)
		self.epochs_slider.setValue(100)
		epochs_layout.addWidget(self.epochs_slider, 70)
		layout.addLayout(epochs_layout)

		self.stardist_model.clicked.connect(self.rescale_slider)
		self.cellpose_model.clicked.connect(self.rescale_slider)

	def populate_data_frame(self):

		"""
		Add widgets and layout in the POST-PROCESSING frame.
		"""

		grid = QGridLayout(self.data_frame)
		grid.setContentsMargins(30,30,30,30)
		grid.setSpacing(30)

		self.data_lbl = QLabel("DATA")
		self.data_lbl.setStyleSheet("""
			font-weight: bold;
			padding: 0px;
			""")
		grid.addWidget(self.data_lbl, 0, 0, 1, 4, alignment=Qt.AlignCenter)
		self.generate_data_contents()
		grid.addWidget(self.ContentsData, 1, 0, 1, 4, alignment=Qt.AlignTop)

	def populate_model_frame(self):

		"""
		Add widgets and layout in the FEATURES frame.
		"""

		grid = QGridLayout(self.model_frame)
		grid.setContentsMargins(30,30,30,30)
		grid.setSpacing(30)

		self.model_lbl = QLabel("MODEL")
		self.model_lbl.setStyleSheet("""
			font-weight: bold;
			padding: 0px;
			""")
		grid.addWidget(self.model_lbl, 0, 0, 1, 4, alignment=Qt.AlignCenter)

		self.generate_model_panel_contents()
		grid.addWidget(self.ContentsModel, 1, 0, 1, 4, alignment=Qt.AlignTop)


	def generate_data_contents(self):

		self.ContentsData = QFrame()
		layout = QVBoxLayout(self.ContentsData)
		layout.setContentsMargins(0,0,0,0)

		train_data_layout = QHBoxLayout()
		train_data_layout.addWidget(QLabel('Training data: '), 30)
		self.select_data_folder_btn = QPushButton('Choose folder')
		self.select_data_folder_btn.clicked.connect(self.showDialog_dataset)
		self.data_folder_label = QLabel('No folder chosen')
		train_data_layout.addWidget(self.select_data_folder_btn, 35)
		train_data_layout.addWidget(self.data_folder_label, 30)

		self.cancel_dataset = QPushButton()
		self.cancel_dataset.setIcon(icon(MDI6.close,color="black"))
		self.cancel_dataset.clicked.connect(self.clear_dataset)
		self.cancel_dataset.setStyleSheet(self.button_select_all)
		self.cancel_dataset.setIconSize(QSize(20, 20))
		self.cancel_dataset.setVisible(False)
		train_data_layout.addWidget(self.cancel_dataset, 5)


		layout.addLayout(train_data_layout)

		include_dataset_layout = QHBoxLayout()
		include_dataset_layout.addWidget(QLabel('include dataset: '),30)
		self.dataset_cb = QComboBox()
		available_datasets, self.datasets_path = get_segmentation_datasets_list(return_path=True)
		signal_datasets = ['--'] + available_datasets #[d.split('/')[-2] for d in available_datasets]
		self.dataset_cb.addItems(signal_datasets)
		include_dataset_layout.addWidget(self.dataset_cb, 70)
		layout.addLayout(include_dataset_layout)

		augmentation_hbox = QHBoxLayout()
		augmentation_hbox.addWidget(QLabel('augmentation\nfactor: '), 30)
		self.augmentation_slider = QLabeledDoubleSlider()
		self.augmentation_slider.setSingleStep(0.01)
		self.augmentation_slider.setTickInterval(0.01)		
		self.augmentation_slider.setOrientation(1)
		self.augmentation_slider.setRange(1, 5)
		self.augmentation_slider.setValue(2.0)

		augmentation_hbox.addWidget(self.augmentation_slider, 70)
		layout.addLayout(augmentation_hbox)

		validation_split_layout = QHBoxLayout()
		validation_split_layout.addWidget(QLabel('validation split: '),30)
		self.validation_slider = QLabeledDoubleSlider()
		self.validation_slider.setSingleStep(0.01)
		self.validation_slider.setTickInterval(0.01)		
		self.validation_slider.setOrientation(1)
		self.validation_slider.setRange(0,0.9)
		self.validation_slider.setValue(0.2)		
		validation_split_layout.addWidget(self.validation_slider, 70)
		layout.addLayout(validation_split_layout)


	def generate_model_panel_contents(self):
		
		self.ContentsModel = QFrame()
		layout = QVBoxLayout(self.ContentsModel)
		layout.setContentsMargins(0,0,0,0)

		model_type_layout = QHBoxLayout()
		model_type_layout.setContentsMargins(30,5,30,15)
		self.cellpose_model = QRadioButton('Cellpose')
		self.stardist_model = QRadioButton('StarDist')
		self.stardist_model.setChecked(True)
		model_type_layout.addWidget(self.stardist_model,50, alignment=Qt.AlignCenter)
		model_type_layout.addWidget(self.cellpose_model,50, alignment=Qt.AlignCenter)
		layout.addLayout(model_type_layout)

		modelname_layout = QHBoxLayout()
		modelname_layout.addWidget(QLabel('Model name: '), 30)
		self.modelname_le = QLineEdit()
		self.modelname_le.setText(f"Untitled_model_{datetime.today().strftime('%Y-%m-%d')}")
		modelname_layout.addWidget(self.modelname_le, 70)
		layout.addLayout(modelname_layout)

		pretrained_layout = QHBoxLayout()
		pretrained_layout.setContentsMargins(0,0,0,0)
		pretrained_layout.addWidget(QLabel('Pretrained model: '), 30)

		self.browse_pretrained_btn = QPushButton('Choose folder')
		self.browse_pretrained_btn.clicked.connect(self.showDialog_pretrained)
		pretrained_layout.addWidget(self.browse_pretrained_btn, 35)

		self.pretrained_lbl = QLabel('No folder chosen')
		pretrained_layout.addWidget(self.pretrained_lbl, 30)

		self.cancel_pretrained = QPushButton()
		self.cancel_pretrained.setIcon(icon(MDI6.close,color="black"))
		self.cancel_pretrained.clicked.connect(self.clear_pretrained)
		self.cancel_pretrained.setStyleSheet(self.button_select_all)
		self.cancel_pretrained.setIconSize(QSize(20, 20))
		self.cancel_pretrained.setVisible(False)
		pretrained_layout.addWidget(self.cancel_pretrained, 5)

		layout.addLayout(pretrained_layout)

		# recompile_layout = QHBoxLayout()
		# recompile_layout.addWidget(QLabel('Recompile: '), 30)
		# self.recompile_option = QCheckBox()
		# self.recompile_option.setEnabled(False)
		# recompile_layout.addWidget(self.recompile_option, 70)
		# layout.addLayout(recompile_layout)

		self.max_nbr_channels = 5
		self.ch_norm = ChannelNormGenerator(self, mode='channels')
		layout.addLayout(self.ch_norm)

		spatial_calib_layout = QHBoxLayout()
		spatial_calib_layout.addWidget(QLabel('input spatial\ncalibration'), 30)
		parent_pxtoum = f"{self.parent_window.parent_window.PxToUm}"
		self.spatial_calib_le = QLineEdit(parent_pxtoum.replace('.',','))
		self.spatial_calib_le.setPlaceholderText('e.g. 0.1 µm per pixel')
		self.spatial_calib_le.setValidator(self.onlyFloat)
		spatial_calib_layout.addWidget(self.spatial_calib_le, 70)
		layout.addLayout(spatial_calib_layout)



	def activate_train_btn(self):
		if self.spatial_calib_le.text()=='':
			self.submit_btn.setEnabled(False)
		elif not np.all([cb.currentText()=='--' for cb in self.ch_norm.channel_cbs]):
			self.submit_btn.setEnabled(True)

	def rescale_slider(self):
		if self.stardist_model.isChecked():
			self.epochs_slider.setRange(1,500)
			self.lr_le.setText('0,0003')
		else:
			self.epochs_slider.setRange(1,10000)
			self.lr_le.setText('0,01')


	def showDialog_pretrained(self):

		try:
			self.cancel_pretrained.click()
		except:
			pass

		self.pretrained_model = None
		self.pretrained_model = QFileDialog.getExistingDirectory(
						self, "Open Directory",
						os.sep.join([self.soft_path, 'celldetective', 'models', f'segmentation_generic','']),
						QFileDialog.ShowDirsOnly | QFileDialog.DontResolveSymlinks,
						)
		
		if self.pretrained_model=='':
			return None
		
		if self.pretrained_model is None:
			return None

		else:
			self.pretrained_model = self.pretrained_model.replace('\\','/')
			self.pretrained_model = rf"{self.pretrained_model}"
			
			subfiles = glob(os.sep.join([self.pretrained_model,"*"]))
			subfiles = [s.replace('\\','/') for s in subfiles]
			subfiles = [rf"{s}" for s in subfiles]

			if "/".join([self.pretrained_model,"config_input.json"]) in subfiles:
				self.load_pretrained_config()
				self.pretrained_lbl.setText(self.pretrained_model.split("/")[-1])
				self.cancel_pretrained.setVisible(True)
				#self.recompile_option.setEnabled(True)
				self.modelname_le.setText(f"{self.pretrained_model.split('/')[-1]}_{datetime.today().strftime('%Y-%m-%d')}")
			else:
				self.pretrained_model = None
				self.pretrained_lbl.setText('No folder chosen')	
				#self.recompile_option.setEnabled(False)	
				self.cancel_pretrained.setVisible(False)
				return None

		self.seg_folder = self.pretrained_model.split('/')[-2]
		self.model_name = self.pretrained_model.split('/')[-1]
		if self.model_name.startswith('CP') and self.seg_folder=='segmentation_generic':

			self.diamWidget = CellposeParamsWidget(self, model_name=self.model_name)
			self.diamWidget.show()

	def set_cellpose_scale(self):

		scale = self.parent_window.parent_window.PxToUm * float(self.diamWidget.diameter_le.text().replace(',','.')) / 30.0
		if self.model_name=="CP_nuclei":
			scale = self.parent_window.parent_window.PxToUm * float(self.diamWidget.diameter_le.text().replace(',','.')) / 17.0
		self.spatial_calib_le.setText(str(scale).replace('.',','))

		for k in range(len(self.diamWidget.cellpose_channel_cb)):
			ch = self.diamWidget.cellpose_channel_cb[k].currentText()
			idx = self.ch_norm.channel_cbs[k].findText(ch)
			self.ch_norm.channel_cbs[k].setCurrentIndex(idx)

		self.diamWidget.close()


	def showDialog_dataset(self):

		self.dataset_folder = QFileDialog.getExistingDirectory(
						self, "Open Directory",
						self.exp_dir,
						QFileDialog.ShowDirsOnly | QFileDialog.DontResolveSymlinks,
						)
		if self.dataset_folder is not None:

			subfiles = glob(self.dataset_folder+os.sep+"*.tif")
			if len(subfiles)>0:
				print(f'found {len(subfiles)} files in folder')
				self.data_folder_label.setText(self.dataset_folder[:16]+'...')
				self.data_folder_label.setToolTip(self.dataset_folder)
				self.cancel_dataset.setVisible(True)
			else:
				self.data_folder_label.setText('No folder chosen')
				self.data_folder_label.setToolTip('')
				self.dataset_folder = None
				self.cancel_dataset.setVisible(False)

	def clear_pretrained(self):
		
		self.pretrained_model = None
		self.pretrained_lbl.setText('No folder chosen')
		for i in range(len(self.ch_norm.channel_cbs)):
			self.ch_norm.channel_cbs[i].setEnabled(True)
			self.ch_norm.normalization_mode_btns[i].setEnabled(True)
			self.ch_norm.normalization_max_value_le[i].setEnabled(True)
			self.ch_norm.normalization_min_value_le[i].setEnabled(True)
			self.ch_norm.normalization_clip_btns[i].setEnabled(True)
			self.ch_norm.normalization_min_value_lbl[i].setEnabled(True)
			self.ch_norm.normalization_max_value_lbl[i].setEnabled(True)
		self.ch_norm.add_col_btn.setEnabled(True)

		self.cancel_pretrained.setVisible(False)
		self.modelname_le.setText(f"Untitled_model_{datetime.today().strftime('%Y-%m-%d')}")

	def clear_dataset(self):

		self.dataset_folder = None
		self.data_folder_label.setText('No folder chosen')
		self.data_folder_label.setToolTip('')
		self.cancel_dataset.setVisible(False)

	def load_stardist_train_config(self):
		
		config = os.sep.join([self.pretrained_model,"config.json"])
		if os.path.exists(config):
			with open(config, 'r') as f:
				config = json.load(f)
				if 'train_batch_size' in config:
					bs = config['train_batch_size']
					self.bs_le.setText(str(bs).replace('.',','))
				if 'train_learning_rate' in config:
					lr = config['train_learning_rate']
					self.lr_le.setText(str(lr).replace('.',','))

	def load_pretrained_config(self):

		f = open(os.sep.join([self.pretrained_model,"config_input.json"]))
		data = json.load(f)
		channels = data["channels"]
		self.seg_folder = self.pretrained_model.split('/')[-2]
		self.model_name = self.pretrained_model.split('/')[-1]
		if self.model_name.startswith('CP') and self.seg_folder=='segmentation_generic':
			channels = ['brightfield_channel', 'live_nuclei_channel']
			if self.model_name=="CP_nuclei":
				channels = ['live_nuclei_channel', 'None']
		if self.model_name.startswith('SD') and self.seg_folder=='segmentation_generic':
			channels = ['live_nuclei_channel']
			if self.model_name=="SD_versatile_he":
				channels = ["H&E_1","H&E_2","H&E_3"]

		normalization_percentile = data['normalization_percentile']
		normalization_clip = data['normalization_clip']
		normalization_values = data['normalization_values']
		spatial_calib = data['spatial_calibration']
		model_type = data['model_type']
		if model_type=='stardist':
			self.stardist_model.setChecked(True)
			self.cellpose_model.setChecked(False)
			self.load_stardist_train_config()
		else:
			self.stardist_model.setChecked(False)
			self.cellpose_model.setChecked(True)			

		for c,cb in zip(channels, self.ch_norm.channel_cbs):
			index = cb.findText(c)
			cb.setCurrentIndex(index)

		for i in range(len(channels)):

			to_clip = normalization_clip[i]
			if self.ch_norm.clip_option[i] != to_clip:
				self.ch_norm.normalization_clip_btns[i].click()

			use_percentile = normalization_percentile[i]
			if self.ch_norm.normalization_mode[i] != use_percentile:
				self.ch_norm.normalization_mode_btns[i].click()

			self.ch_norm.normalization_min_value_le[i].setText(str(normalization_values[i][0]))
			self.ch_norm.normalization_max_value_le[i].setText(str(normalization_values[i][1]))
			

		if len(channels)<len(self.ch_norm.channel_cbs):
			for k in range(len(self.ch_norm.channel_cbs)-len(channels)):
				self.ch_norm.channel_cbs[len(channels)+k].setCurrentIndex(0)
				self.ch_norm.channel_cbs[len(channels)+k].setEnabled(False)
				self.ch_norm.normalization_mode_btns[len(channels)+k].setEnabled(False)
				self.ch_norm.normalization_max_value_le[len(channels)+k].setEnabled(False)
				self.ch_norm.normalization_min_value_le[len(channels)+k].setEnabled(False)
				self.ch_norm.normalization_min_value_lbl[len(channels)+k].setEnabled(False)
				self.ch_norm.normalization_max_value_lbl[len(channels)+k].setEnabled(False)
				self.ch_norm.normalization_clip_btns[len(channels)+k].setEnabled(False)
		self.ch_norm.add_col_btn.setEnabled(False)

		self.spatial_calib_le.setText(str(spatial_calib).replace('.',','))

	def adjustScrollArea(self):
		
		"""
		Auto-adjust scroll area to fill space 
		(from https://stackoverflow.com/questions/66417576/make-qscrollarea-use-all-available-space-of-qmainwindow-height-axis)
		"""

		step = 5
		while self.scroll_area.verticalScrollBar().isVisible() and self.height() < self.maximumHeight():
			self.resize(self.width(), self.height() + step)

	def prep_model(self):

		model_name = self.modelname_le.text()
		pretrained_model = self.pretrained_model

		channels = []
		for i in range(len(self.ch_norm.channel_cbs)):
			channels.append(self.ch_norm.channel_cbs[i].currentText())

		slots_to_keep = np.where(np.array(channels)!='--')[0]
		while '--' in channels:
			channels.remove('--')

		norm_values = np.array([[float(a.replace(',','.')),float(b.replace(',','.'))] for a,b in zip([l.text() for l in self.ch_norm.normalization_min_value_le],
											[l.text() for l in self.ch_norm.normalization_max_value_le])])
		norm_values = norm_values[slots_to_keep]
		norm_values = [list(v) for v in norm_values]

		clip_values = np.array(self.ch_norm.clip_option)
		clip_values = list(clip_values[slots_to_keep])
		clip_values = [bool(c) for c in clip_values]

		normalization_mode = np.array(self.ch_norm.normalization_mode)
		normalization_mode = list(normalization_mode[slots_to_keep])
		normalization_mode = [bool(m) for m in normalization_mode]

		data_folders = []
		if self.dataset_folder is not None:
			data_folders.append(self.dataset_folder)
		if self.dataset_cb.currentText()!='--':
			dataset = locate_segmentation_dataset(self.dataset_cb.currentText()) #glob(self.soft_path+'/celldetective/datasets/signals/*/')[self.dataset_cb.currentIndex()-1]
			data_folders.append(dataset)

		aug_factor = round(self.augmentation_slider.value(),2)
		val_split = round(self.validation_slider.value(),2)
		if self.stardist_model.isChecked():
			model_type = 'stardist'
		else:
			model_type = 'cellpose'

		try:
			lr = float(self.lr_le.text().replace(',','.'))
		except:
			msgBox = QMessageBox()
			msgBox.setIcon(QMessageBox.Warning)
			msgBox.setText("Invalid value encountered for the learning rate.")
			msgBox.setWindowTitle("Warning")
			msgBox.setStandardButtons(QMessageBox.Ok)
			returnValue = msgBox.exec()
			if returnValue == QMessageBox.Ok:
				return None			
		
		bs = int(self.bs_le.text())
		epochs = self.epochs_slider.value()
		spatial_calib = float(self.spatial_calib_le.text().replace(',','.'))

		training_instructions = {'model_name': model_name,'model_type': model_type, 'pretrained': pretrained_model, 'spatial_calibration': spatial_calib, 'channel_option': channels, 'normalization_percentile': normalization_mode,
		'normalization_clip': clip_values,'normalization_values': norm_values, 'ds': data_folders, 'augmentation_factor': aug_factor, 'validation_split': val_split,
		'learning_rate': lr, 'batch_size': bs, 'epochs': epochs}

		print(training_instructions)

		model_folder = os.sep.join([self.software_models_dir,model_name, ''])
		print(model_folder)
		if not os.path.exists(model_folder):
			os.mkdir(model_folder)

		training_instructions.update({'target_directory': self.software_models_dir})

		print(f"Set of instructions: {training_instructions}")
		with open(model_folder+"training_instructions.json", 'w') as f:
			json.dump(training_instructions, f, indent=4)
		
		train_segmentation_model(model_folder+"training_instructions.json", use_gpu=self.parent_window.parent_window.parent_window.use_gpu)

		# self.parent.refresh_signal_models()


from setuptools import setup
import setuptools
import pip
import os

from pip._internal.req import parse_requirements
from pathlib import Path
this_directory = Path(__file__).parent

links = []
requires = []

requirements = parse_requirements('requirements.txt', session='hack')
requirements = list(requirements) 
try:
    requirements = [str(ir.req) for ir in requirements]
except:
    requirements = [str(ir.requirement) for ir in requirements]

setup(name='celldetective',
			version='1.2.2.post2',
			description='description',
			long_description=(this_directory / "README.md").read_text(),
			#long_description=open('README.rst',encoding="utf8").read(),
			long_description_content_type='text/markdown',
			url='http://github.com/remyeltorro/celldetective',
			author='Rémy Torro',
			author_email='remy.torro@inserm.fr',
			license='GPL-3.0',
			packages=setuptools.find_packages(),
			zip_safe=False,
			package_data={'celldetective': ['*',os.sep.join(['scripts','*']),os.sep.join(['gui','*']),os.sep.join(['gui','help','*']),os.sep.join(['models','*','*','*']),os.sep.join(['models','*']),os.sep.join(['models','*','*']),os.sep.join(['icons','*']),os.sep.join(['links','*']),os.sep.join(['datasets','*']), os.sep.join(['datasets','*','*'])]},
			entry_points = {
				'console_scripts': [
					'celldetective = celldetective.__main__:main'],
			},
			install_requires = requirements,
			#dependency_links = links
			)




import unittest
import numpy as np
import pandas as pd
from celldetective.tracking import filter_by_endpoints, extrapolate_tracks, filter_by_tracklength, interpolate_time_gaps

class TestTrackFilteringByEndpoint(unittest.TestCase):

	@classmethod
	def setUpClass(self):
		self.tracks = pd.DataFrame([{"TRACK_ID": 0., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 15},
									{"TRACK_ID": 0., "FRAME": 1, "POSITION_X": 15, "POSITION_Y": 10},
									{"TRACK_ID": 0., "FRAME": 2, "POSITION_X": 30, "POSITION_Y": 5},
									{"TRACK_ID": 0., "FRAME": 3, "POSITION_X": 40, "POSITION_Y": 0},
									{"TRACK_ID": 1., "FRAME": 1, "POSITION_X": 5, "POSITION_Y": 20},
									{"TRACK_ID": 1., "FRAME": 2, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 1, "POSITION_X": 10, "POSITION_Y": 25}

									])

	def test_filter_not_in_last(self):
		self.filtered_tracks = filter_by_endpoints(self.tracks, remove_not_in_first=False, remove_not_in_last=True)
		track_ids = list(self.filtered_tracks['TRACK_ID'].unique())
		self.assertEqual(track_ids,[0.])

	def test_filter_not_in_first(self):
		self.filtered_tracks = filter_by_endpoints(self.tracks, remove_not_in_first=True, remove_not_in_last=False)
		track_ids = list(self.filtered_tracks['TRACK_ID'].unique())
		self.assertEqual(track_ids,[0.,2.])

	def test_no_filter_does_nothing(self):
		self.filtered_tracks = filter_by_endpoints(self.tracks, remove_not_in_first=False, remove_not_in_last=False)
		track_ids = list(self.filtered_tracks['TRACK_ID'].unique())		
		self.assertEqual(track_ids,list(self.tracks['TRACK_ID'].unique()))


class TestTrackFilteringByLength(unittest.TestCase):

	@classmethod
	def setUpClass(self):
		self.tracks = pd.DataFrame([{"TRACK_ID": 0., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 15},
									{"TRACK_ID": 0., "FRAME": 1, "POSITION_X": 15, "POSITION_Y": 10},
									{"TRACK_ID": 0., "FRAME": 2, "POSITION_X": 30, "POSITION_Y": 5},
									{"TRACK_ID": 0., "FRAME": 3, "POSITION_X": 40, "POSITION_Y": 0},
									{"TRACK_ID": 1., "FRAME": 1, "POSITION_X": 5, "POSITION_Y": 20},
									{"TRACK_ID": 1., "FRAME": 2, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 1, "POSITION_X": 10, "POSITION_Y": 25}
									])

	def test_filter_by_tracklength_of_zero(self):
		self.filtered_tracks = filter_by_tracklength(self.tracks, minimum_tracklength=0)
		track_ids = list(self.filtered_tracks['TRACK_ID'].unique())
		self.assertEqual(track_ids,[0.,1.,2.])

	def test_filter_by_tracklength_of_three(self):
		self.filtered_tracks = filter_by_tracklength(self.tracks, minimum_tracklength=3)
		track_ids = list(self.filtered_tracks['TRACK_ID'].unique())
		self.assertEqual(track_ids,[0.])


class TestTrackInterpolation(unittest.TestCase):

	@classmethod
	def setUpClass(self):

		self.tracks = pd.DataFrame([{"TRACK_ID": 0., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 15},
									{"TRACK_ID": 0., "FRAME": 1, "POSITION_X": 15, "POSITION_Y": 10},
									#{"TRACK_ID": 0., "FRAME": 2, "POSITION_X": 20, "POSITION_Y": 5},
									{"TRACK_ID": 0., "FRAME": 3, "POSITION_X": 25, "POSITION_Y": 0},
									{"TRACK_ID": 1., "FRAME": 1, "POSITION_X": 5, "POSITION_Y": 20},
									{"TRACK_ID": 1., "FRAME": 2, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 25},
									#{"TRACK_ID": 2., "FRAME": 1, "POSITION_X": 5, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 2, "POSITION_X": 0, "POSITION_Y": 25}
									])
		self.tracks_real_intep = pd.DataFrame([{"TRACK_ID": 0., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 15},
									{"TRACK_ID": 0., "FRAME": 1, "POSITION_X": 15, "POSITION_Y": 10},
									{"TRACK_ID": 0., "FRAME": 2, "POSITION_X": 20, "POSITION_Y": 5},
									{"TRACK_ID": 0., "FRAME": 3, "POSITION_X": 25, "POSITION_Y": 0},
									{"TRACK_ID": 1., "FRAME": 1, "POSITION_X": 5, "POSITION_Y": 20},
									{"TRACK_ID": 1., "FRAME": 2, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 1, "POSITION_X": 5, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 2, "POSITION_X": 0, "POSITION_Y": 25}
									])


	def test_interpolate_tracks_as_expected(self):
		self.interpolated_tracks = interpolate_time_gaps(self.tracks)
		self.assertTrue(np.array_equal(self.interpolated_tracks.to_numpy(), self.tracks_real_intep.to_numpy(), equal_nan=True))

class TestTrackExtrapolation(unittest.TestCase):

	@classmethod
	def setUpClass(self):

		self.tracks = pd.DataFrame([{"TRACK_ID": 0., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 15},
									{"TRACK_ID": 0., "FRAME": 1, "POSITION_X": 15, "POSITION_Y": 10},
									{"TRACK_ID": 0., "FRAME": 2, "POSITION_X": 20, "POSITION_Y": 5},
									{"TRACK_ID": 0., "FRAME": 3, "POSITION_X": 25, "POSITION_Y": 0},
									{"TRACK_ID": 1., "FRAME": 1, "POSITION_X": 5, "POSITION_Y": 20},
									{"TRACK_ID": 1., "FRAME": 2, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 1, "POSITION_X": 5, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 2, "POSITION_X": 0, "POSITION_Y": 25}
									])
		self.tracks_pre_extrapol = pd.DataFrame([
									{"TRACK_ID": 0., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 15},
									{"TRACK_ID": 0., "FRAME": 1, "POSITION_X": 15, "POSITION_Y": 10},
									{"TRACK_ID": 0., "FRAME": 2, "POSITION_X": 20, "POSITION_Y": 5},
									{"TRACK_ID": 0., "FRAME": 3, "POSITION_X": 25, "POSITION_Y": 0},
									{"TRACK_ID": 1., "FRAME": 0, "POSITION_X": 5, "POSITION_Y": 20},									
									{"TRACK_ID": 1., "FRAME": 1, "POSITION_X": 5, "POSITION_Y": 20},
									{"TRACK_ID": 1., "FRAME": 2, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 1, "POSITION_X": 5, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 2, "POSITION_X": 0, "POSITION_Y": 25}
									])
		self.tracks_post_extrapol = pd.DataFrame([
									{"TRACK_ID": 0., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 15},
									{"TRACK_ID": 0., "FRAME": 1, "POSITION_X": 15, "POSITION_Y": 10},
									{"TRACK_ID": 0., "FRAME": 2, "POSITION_X": 20, "POSITION_Y": 5},
									{"TRACK_ID": 0., "FRAME": 3, "POSITION_X": 25, "POSITION_Y": 0},
									{"TRACK_ID": 1., "FRAME": 1, "POSITION_X": 5, "POSITION_Y": 20},
									{"TRACK_ID": 1., "FRAME": 2, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 1., "FRAME": 3, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 1, "POSITION_X": 5, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 2, "POSITION_X": 0, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 3, "POSITION_X": 0, "POSITION_Y": 25}
									])

		self.tracks_full_extrapol = pd.DataFrame([
									{"TRACK_ID": 0., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 15},
									{"TRACK_ID": 0., "FRAME": 1, "POSITION_X": 15, "POSITION_Y": 10},
									{"TRACK_ID": 0., "FRAME": 2, "POSITION_X": 20, "POSITION_Y": 5},
									{"TRACK_ID": 0., "FRAME": 3, "POSITION_X": 25, "POSITION_Y": 0},
									{"TRACK_ID": 1., "FRAME": 0, "POSITION_X": 5, "POSITION_Y": 20},
									{"TRACK_ID": 1., "FRAME": 1, "POSITION_X": 5, "POSITION_Y": 20},
									{"TRACK_ID": 1., "FRAME": 2, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 1., "FRAME": 3, "POSITION_X": 10, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 0, "POSITION_X": 10, "POSITION_Y": 25},				
									{"TRACK_ID": 2., "FRAME": 1, "POSITION_X": 5, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 2, "POSITION_X": 0, "POSITION_Y": 25},
									{"TRACK_ID": 2., "FRAME": 3, "POSITION_X": 0, "POSITION_Y": 25}
									])


	def test_pre_extrapolate(self):
		self.extrapolated_tracks = extrapolate_tracks(self.tracks,post=False, pre=True)
		self.assertTrue(np.array_equal(self.extrapolated_tracks.to_numpy(), self.tracks_pre_extrapol.to_numpy(), equal_nan=True))

	def test_post_extrapolate(self):
		self.extrapolated_tracks = extrapolate_tracks(self.tracks,post=True, pre=False)
		self.assertTrue(np.array_equal(self.extrapolated_tracks.to_numpy(), self.tracks_post_extrapol.to_numpy(), equal_nan=True))

	def test_full_extrapolate(self):
		self.extrapolated_tracks = extrapolate_tracks(self.tracks,post=True, pre=True)
		self.assertTrue(np.array_equal(self.extrapolated_tracks.to_numpy(), self.tracks_full_extrapol.to_numpy(), equal_nan=True))


if __name__=="__main__":
	unittest.main()


"""
Copright © 2023 Laboratoire Adhesion et Inflammation, Authored by Remy Torro.
"""

import argparse
import os
import json
from glob import glob
import numpy as np
from art import tprint
from celldetective.signals import SignalDetectionModel
from celldetective.io import locate_signal_model

tprint("Train")

parser = argparse.ArgumentParser(description="Train a signal model from instructions.",
								formatter_class=argparse.ArgumentDefaultsHelpFormatter)
parser.add_argument('-c',"--config", required=True,help="Training instructions")

args = parser.parse_args()
process_arguments = vars(args)
instructions = str(process_arguments['config'])

if os.path.exists(instructions):
	with open(instructions, 'r') as f:
		threshold_instructions = json.load(f)
		threshold_instructions.update({'n_channels': len(threshold_instructions['channel_option'])})
		if threshold_instructions['augmentation_factor']>1.0:
			threshold_instructions.update({'augment': True})
		else:
			threshold_instructions.update({'augment': False})
		threshold_instructions.update({'test_split': 0.})
else:
	print('The configuration path is not valid. Abort.')
	os.abort()

all_classes = []
for d in threshold_instructions["ds"]:
	datasets = glob(d+os.sep+"*.npy")
	for dd in datasets:
		data = np.load(dd, allow_pickle=True)
		classes = np.unique([ddd["class"] for ddd in data])
		all_classes.extend(classes)
all_classes = np.unique(all_classes)
print(all_classes,len(all_classes))

n_classes = len(all_classes)

model_params = {k:threshold_instructions[k] for k in ('pretrained', 'model_signal_length', 'channel_option', 'n_channels', 'label') if k in threshold_instructions}
model_params.update({'n_classes': n_classes})

train_params = {k:threshold_instructions[k] for k in ('model_name', 'target_directory', 'channel_option','recompile_pretrained', 'test_split', 'augment', 'epochs', 'learning_rate', 'batch_size', 'validation_split','normalization_percentile','normalization_values','normalization_clip') if k in threshold_instructions}

print(f'model params {model_params}')
print(f'train params {train_params}')

model = SignalDetectionModel(**model_params)
print(threshold_instructions['ds'])
model.fit_from_directory(threshold_instructions['ds'], **train_params)


# if neighborhood of interest in training instructions, write it in config!
if 'neighborhood_of_interest' in threshold_instructions:
	if threshold_instructions['neighborhood_of_interest'] is not None:
		
		model_path = locate_signal_model(threshold_instructions['model_name'], path=None, pairs=True)
		complete_path = model_path #+model
		complete_path = rf"{complete_path}"
		model_config_path = os.sep.join([complete_path,'config_input.json'])
		model_config_path = rf"{model_config_path}"

		f = open(model_config_path)
		config = json.load(f)
		config.update({'neighborhood_of_interest': threshold_instructions['neighborhood_of_interest'], 'reference_population': threshold_instructions['reference_population'], 'neighbor_population': threshold_instructions['neighbor_population']})
		json_string = json.dumps(config)
		with open(model_config_path, 'w') as outfile:
			outfile.write(json_string)

print('Done.')